import sys
import json
import time
import uuid
import traceback
import datetime
import inspect

import openai

from .sessions import start, end, log_event

class HoneyHiveTraceContextManager:
    def __init__(self, tracer):
        self.tracer = tracer
        self.event_id = tracer.event_id
        self.original_trace_function = None
        self.first_call_value = None
        self.last_return_value = None
        self.recorded_exception = None
        self.event_data = {
            "event_name": self.tracer.event_name,
            "input": {},
            "output": "",
            "error": None,
            "metadata": {},
            "duration": 0
        }
        self.start_time = None
        self.end_time = None
    
    def __enter__(self):
        self.original_trace_function = sys.gettrace()
        sys.settrace(self.general_trace_call)
        self.start_time = time.time()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.end_time = time.time()
        sys.settrace(self.original_trace_function)
        self.prepare_input()
        self.prepare_output()
        self.prepare_error(exc_type, exc_val)
        self.tracer.log_event_with_data(self.event_data)
    
    def prepare_input(self):
        # prepare input, output, metadata, duration, error
        if 'kwargs' in self.first_call_value:
            temp = dict(self.first_call_value['kwargs'])
            del self.first_call_value['kwargs']
            for key, value in temp.items():
                # if value is an object, convert to dict
                if hasattr(value, '__dict__'):
                    self.first_call_value[key] = value.__dict__
                else:
                    self.first_call_value[key] = value
        if 'args' in self.first_call_value:
            temp = self.first_call_value['args']
            # if args is a tuple, convert to list
            if isinstance(temp, tuple):
                temp = list(temp)
            self.first_call_value['args'] = temp
        
        # drop anything that's non-JSON serializable
        new_call_value = {}
        for key, value in self.first_call_value.items():
            try:
                json.dumps(value)
                new_call_value[key] = value
            except TypeError:
                pass
        if self.tracer.input != None and self.tracer.input != {}:
            # put it in metadata
            self.event_data['metadata']['input'] = new_call_value
        else:
            self.event_data['input'] = new_call_value
        ##print("left call for set context")
    
    def prepare_output(self):
        if self.last_return_value == None:
            return
        if isinstance(self.last_return_value, dict):
            for key, value in self.last_return_value.items():
                if key == self.tracer.output_field:
                    self.event_data['output'] = str(value)
                else:
                    if self.event_data['metadata'] == None:
                        self.event_data['metadata'] = {}
                    # if value has __dict__ attribute, convert to dict
                    if hasattr(value, '__dict__'):
                        self.event_data['metadata'][key] = value.__dict__
                    else:
                        self.event_data['metadata'][key] = value
        else:
            try:
                self.event_data['output'] = str(self.last_return_value)
            except:
                pass
            finally:
                if self.event_data['metadata'] == None:
                    self.event_data['metadata'] = {}
                if self.event_data['output'] == "":
                    self.event_data['metadata']['output'] = self.last_return_value
        
        self.event_data['duration'] = (self.end_time - self.start_time) * 1000
    
    def prepare_error(self, ex_type, ex_value):
        if ex_type == None:
            return
        ex_type = str(ex_type)
        ##print(self.recorded_exception)
        self.event_data['error'] = ex_type
        if self.event_data['metadata'] == None:
            self.event_data['metadata'] = {}
        self.event_data['metadata']["error_message"] = str(ex_value)
        self.event_data['duration'] = (self.end_time - self.start_time) * 1000
    
    def general_trace_call(self, frame, event, arg):
        if event == 'call' and self.first_call_value == None:
            self.first_call_value = frame.f_locals
            if self.event_data['event_name'] == "":
                self.event_data['event_name'] = frame.f_code.co_name
        elif event == 'return':
            self.last_return_value = arg

        return self.general_trace_call
    
class HoneyHiveTracer:
    def __init__(self, project, name, source, user_properties={}, show_trace=False):

        honeyhive_session = start(
            project=project,
            source=source,
            session_name=name,
            user_properties=user_properties
        )
        self.events = []
        self.source = source
        self.session_id = honeyhive_session.session_id
        self.event_id = None
        self.parent_id = None
        self.event_type = "model"
        self.event_name = ""
        self.config = {
            "provider": "",
            "endpoint": ""
        }
        self.children = []
        self.input = {}
        self.output = ""
        self.source = source
        self.error = None
        self.duration = 0
        self.user_properties = user_properties
        self.metadata = {}
        self.context_set = False
        self.context_set_properly = False
        self.context_function_name = ""
        self.context_file_name = ""
        self.start_time = 0
        self.end_time = 0
        self.call_value = None
        self.return_value = None
        self.show_trace = show_trace

    def trace_calls(self, frame, event, arg):
        function_name = frame.f_code.co_name
        file_name = frame.f_code.co_filename
        if event == 'call':
            if function_name == "create" and file_name == inspect.getfile(openai.Completion):
                self.start_event()
                self.config['provider'] = "openai"
                self.config['endpoint'] = "completion"
                self.event_type = "model"
                self.call_value = frame.f_locals['kwargs']
                for key, value in self.call_value.items():
                    if key != "prompt":
                        self.config[key] = value
                    elif self.input == None or self.input == {}:
                        self.input = value
            elif function_name == "create" and file_name == inspect.getfile(openai.ChatCompletion):
                self.start_event()
                
                self.config['provider'] = "openai"
                self.config['endpoint'] = "chat"
                self.event_type = "model"
                self.call_value = frame.f_locals['kwargs']
                for key, value in self.call_value.items():
                    if key != "messages":
                        self.config[key] = value
                    elif self.input == None or self.input == {}:
                        self.input = value
            elif self.context_set and (function_name == self.context_function_name and file_name == self.context_file_name):
                self.start_event(self.event_type)
                self.context_set_properly = True
                ##print("inside call for set context")
                ##print(function_name)
                ##print(file_name)
                ##print(frame.f_locals)

                self.call_value = frame.f_locals
                if 'kwargs' in self.call_value:
                    temp = dict(self.call_value['kwargs'])
                    del self.call_value['kwargs']
                    for key, value in temp.items():
                        self.call_value[key] = value
                if 'args' in self.call_value:
                    temp = self.call_value['args']
                    # if args is a tuple, convert to list
                    if isinstance(temp, tuple):
                        temp = list(temp)
                    self.call_value['args'] = temp
                
                # drop anything that's non-JSON serializable
                new_call_value = {}
                for key, value in self.call_value.items():
                    try:
                        json.dumps(value)
                        new_call_value[key] = value
                    except TypeError:
                        pass
                if self.input:
                    # put it in metadata
                    self.metadata['input'] = new_call_value
                else:
                    self.input = new_call_value
                ##print("left call for set context")
        elif event == 'exception':
            if function_name == "create" and (file_name == inspect.getfile(openai.ChatCompletion) or file_name == inspect.getfile(openai.Completion)):
                # from the exception, get the message and type
                ##print("exception")
                ex_type, ex_value, ex_traceback = arg
                ex_type = str(ex_type).split(".")[2][:-2]
                self.error = ex_type
                self.metadata["error_message"] = str(ex_value)
                self.log_event()
            elif self.context_set and self.context_set_properly and (function_name == self.context_function_name and file_name == self.context_file_name):
                ##print("inside exception for set context")
                ##print(function_name)
                ##print(file_name)
                ex_type, ex_value, ex_traceback = arg
                ex_type = str(ex_type)
                self.error = ex_type
                self.metadata["error_message"] = str(ex_value)
                # print out the traceback
                traceback.print_tb(ex_traceback)
                self.log_event()
                ##print("left exception for set context")
        elif event == 'return':
            if function_name == "create" and file_name == inspect.getfile(openai.Completion):
                self.end_time = time.time()
                self.return_value = arg
                self.duration = (self.end_time - self.start_time) * 1000
                if self.return_value:
                    for key, value in self.return_value.items():
                        if key == "model":
                            self.config[key] = value
                        elif key == "created_at":
                            # turn created_at from utc number to isoformat
                            self.metadata[key] = datetime.fromtimestamp(value).isoformat()
                        elif key == "choices":
                            self.output = value[0].text
                        elif key == "usage":
                            for key, value in value.items():
                                self.metadata[key] = value
                if arg:
                    arg['event_id'] = self.event_id
                self.log_event()
            elif function_name == "create" and file_name == inspect.getfile(openai.ChatCompletion):
                self.end_time = time.time()
                self.return_value = arg
                self.duration = (self.end_time - self.start_time) * 1000
                if self.return_value:
                    for key, value in self.return_value.items():
                        if key == "model":
                            self.config[key] = value
                        elif key == "created_at":
                            # turn created_at from utc number to isoformat
                            self.config[key] = datetime.fromtimestamp(value).isoformat()
                        elif key == "choices":
                            self.output = value[0].message.content
                        elif key == "usage":
                            for key, value in value.items():
                                self.metadata[key] = value
                self.log_event()
            elif self.context_set and self.context_set_properly and (function_name == self.context_function_name and file_name == self.context_file_name):
                ##print("inside return for set context")
                ##print(function_name)
                ##print(file_name)
                self.return_value = arg
                self.end_time = time.time()
                self.duration = (self.end_time - self.start_time) * 1000
                ##print(self.return_value)
                # check if return_value is of type dict
                if self.return_value:
                    if isinstance(self.return_value, dict):
                        for key, value in self.return_value.items():
                            if key == self.output_field:
                                self.output = str(value)
                            else:
                                self.metadata[key] = value
                    else:
                        self.output = self.return_value
                ##print("left return for set context")
                self.log_event()

        return self.trace_calls
    
    def start_event(self, event_type="model", parent_id=None):
        if self.event_id == None:
            self.event_id = str(uuid.uuid4())
        self.start_time = time.time()
        self.event_type = event_type
        self.output = ""
        self.duration = 0
        self.call_value = None
        self.return_value = None
        self.error = None
        self.metadata = {}
        if parent_id:
            self.parent_id = parent_id
        else:
            self.parent_id = self.session_id
    
    def log_event(self):
        #print("Logging event")
        if self.show_trace:
            self.print_event()

        log_event(
            session_id=self.session_id,
            event_id=self.event_id,
            event_name=self.event_name,
            event_type=self.event_type,
            config=self.config,
            input=self.input,
            output=self.output,
            error=self.error,
            duration=self.duration,
            metadata=self.metadata,
            user_properties=self.user_properties,
            children=self.children,
            parent_id=self.parent_id if self.parent_id else self.session_id
        )

        # reset the event
        self.event_id = None
        self.event_type = "model"
        self.event_name = ""
        self.config = {}
        self.children = []
        self.input = {}
        self.output = ""
        self.error = None
        self.duration = 0
        self.user_properties = {}
        self.metadata = {}
        self.start_time = 0
        self.end_time = 0
        self.call_value = None
        self.return_value = None
        self.context_set = False
        self.output_field = ""
    
    def log_event_with_data(self, event_data):
        # find all the keys in event_data that are not None
        # and add them to self
        for key, value in event_data.items():
            if value != None:
                # check if there's already a value for the key
                # if there is, don't overwrite it
                if getattr(self, key) == None:
                    setattr(self, key, value)
                else:
                    if isinstance(value, dict):
                        getattr(self, key).update(value)
                    else:
                        setattr(self, key, value)
        
        # check if "provider" and "endpoint" are in config
        if 'provider' in self.config and 'endpoint' in self.config:
            if self.config['provider'] == "openai":
                if self.config['endpoint'] == "completion":
                    self.output = event_data['metadata']['choices'][0]['text']
                elif self.config['endpoint'] == "chat":
                    self.output = event_data['metadata']['choices'][0]['message']['content']
        self.log_event()

    def end_session(self):
        end(
            session_id=self.session_id
        )
    
    def set_model_context(self, event_name, input, func, prompt_template="", chat_template=[], output_field=""):
        #self.start_event()
        self.event_type = "model"
        self.event_name = event_name
        if chat_template != []:
            self.config['chat_template'] = chat_template
        if prompt_template != "":
            self.config['prompt_template'] = prompt_template
        self.input = input
        self.output_field = output_field
        if self.event_id == None:
            self.event_id = str(uuid.uuid4())
        self.context_set = True
        self.context_function_name = func.__name__
        self.context_file_name = inspect.getfile(func)
        return self.event_id

    def set_tool_context(self, event_name, description, func, config={}, output_field=""):
        #self.start_event()
        self.event_type = "tool"
        self.event_name = event_name
        self.config = config
        self.config['description'] = description
        self.output_field = output_field
        if self.event_id == None:
            self.event_id = str(uuid.uuid4())
        self.context_set = True
        self.context_function_name = func.__name__
        self.context_file_name = inspect.getfile(func)
        #print("set tool context")
        #print(self.context_function_name)
        #print(self.context_file_name)
        return self.event_id
    
    def set_session_context(self, user_properties={}, metadata={}):
        self.user_properties = user_properties
        self.metadata = metadata
    
    def print_event(self):
        print("event")
        print("Event ID: " + self.event_id)
        print("Parent ID: " + self.parent_id)
        print("Event Type: " + self.event_type)
        print("Event Name: " + self.event_name)
        print("Config:")
        print(self.config)
        print("Input:")
        print(self.input)
        print("Output: " + self.output)
        print("Error: " + str(self.error))
        print("Duration: " + str(self.duration))
        print("Metadata:")
        print(self.metadata)
        print("User Properties:")
        print(self.user_properties)
        print("Children:")
        print(self.children)
        print("----")

    # def print_trace(self):
    #     # call_value and return_value are dictionaries
    #     # print the key value pairs on separate lines
    #     for key, value in self.call_value.items():
    #         #print(key)
    #         #print(str(value))
    #         #print()
    #     #print("------------------")
    #     for key, value in self.return_value.items():
    #         #print(key)
    #         #print(str(value))
    #         #print()
    #     #print("------------------")
    #     #print("Duration: " + str((self.end_time - self.start_time) * 1000) + " ms")

    def tool(
        self,
        event_name,
        description="",
        config={},
        output_field=""):
        self.start_event(event_type="tool")
        self.event_name = event_name
        self.config = config
        self.config['description'] = description
        self.output_field = output_field

        self.trace_context_manager = HoneyHiveTraceContextManager(self)
        return self.trace_context_manager

    def model(
        self,
        event_name,
        input,
        config={},
        description="",
        output_field="",
        prompt_template="",
        chat_template={}
    ):
        self.start_event(event_type="model")
        self.event_name = event_name
        self.config = config
        self.config['description'] = description
        self.input = input
        self.output_field = output_field
        if chat_template != {}:
            self.config['chat_template'] = chat_template
        if prompt_template != "":
            self.config['prompt_template'] = prompt_template
        
        self.trace_context_manager = HoneyHiveTraceContextManager(self)
        return self.trace_context_manager

    def chat_openai(
        self,
        event_name,
        input,
        config={},
        description="",
        output_field="",
        chat_template={}
    ):
        self.start_event(event_type="model")
        self.event_name = event_name
        self.config = config
        self.config['description'] = description
        self.config['provider'] = "openai"
        self.config['endpoint'] = "chat"
        self.input = input
        self.output_field = output_field
        if chat_template != {}:
            self.config['chat_template'] = chat_template
        
        self.trace_context_manager = HoneyHiveTraceContextManager(self)
        return self.trace_context_manager
    
    def completion_openai(
        self,
        event_name,
        input,
        config={},
        description="",
        output_field="",
        prompt_template=""
    ):
        self.start_event(event_type="model")
        self.event_name = event_name
        self.config = config
        self.config['description'] = description
        self.config['provider'] = "openai"
        self.config['endpoint'] = "completion"
        self.input = input
        self.output_field = output_field
        if prompt_template != "":
            self.config['prompt_template'] = prompt_template
        
        self.trace_context_manager = HoneyHiveTraceContextManager(self)
        return self.trace_context_manager
    
    @staticmethod
    @DeprecationWarning
    def trace_chain(project, name, source, user_properties={}):
        def decorator(func):
            def wrapper(*args, **kwargs):
                trace = HoneyHiveTracer(project, name, source, user_properties)
                sys.settrace(trace.trace_calls)
                kwargs['honeyhive_tracer'] = trace
                result = func(*args, **kwargs)
                sys.settrace(None)
                trace.end_session()
                return result
            return wrapper
        return decorator

__all__ = [
    "HoneyHiveTracer"
]


from typing import Optional
from io import BufferedReader
import json

from honeyhive.sdk.init import honeyhive_client
from honeyhive.api.models.datasets import (
    UploadDataset,
    DatasetResponse,
    ListDatasetResponse,
    FetchDataset,
)
from honeyhive.api.models.utils import DeleteResponse


def get_datasets(
    name: Optional[str] = None,
    project: Optional[str] = None,
    prompt: Optional[str] = None,
    purpose: Optional[str] = None,
) -> ListDatasetResponse:
    """Get all datasets"""
    client = honeyhive_client()
    return client.get_datasets(
        name=name, task=project, prompt=prompt, purpose=purpose
    )


def get_dataset(name: str) -> DatasetResponse:
    """Get a dataset"""
    client = honeyhive_client()
    return client.get_dataset(name=name)


def create_dataset(
    name: str,
    project: str,
    purpose: str = None,
    file: BufferedReader = None,
    prompt: Optional[str] = None,
    description: Optional[str] = None,
) -> DatasetResponse:
    """Create a dataset"""
    client = honeyhive_client()

    file_contents = file.read()
    file.close()

    # validate the file_contents are a list of json objects
    try:
        file_contents = json.loads(file_contents)
    except json.JSONDecodeError:
        raise ValueError("File contents must be a list of json objects")

    return client.create_dataset(
        dataset=UploadDataset(
            name=name,
            task=project,
            prompt=prompt,
            purpose=purpose,
            description=description,
            file=file_contents,
        )
    )


def delete_dataset(name: str) -> DeleteResponse:
    """Delete a dataset"""
    client = honeyhive_client()
    return client.delete_dataset(name=name)


__all__ = ["get_datasets", "create_dataset", "get_dataset", "delete_dataset"]



from typing import Optional, Any, Dict, List
from honeyhive.api.models.prompts import (
    PromptCreationQuery,
    PromptQuery,
    PromptUpdateQuery,
    PromptResponse,
    ListPromptResponse,
)
from honeyhive.api.models.utils import DeleteResponse
from honeyhive.sdk.init import honeyhive_client


def get_prompts(
    project: Optional[str] = None, prompt_id: Optional[str] = None
) -> List[PromptResponse]:
    """Get all prompts"""
    client = honeyhive_client()
    return client.get_prompts(task=project, name=prompt_id)


def get_deployed_prompts(
    project: Optional[str] = None,
) -> List[PromptResponse]:
    """Get all deployed prompts"""
    client = honeyhive_client()
    prompts = client.get_prompts(task=project)
    return [prompt for prompt in prompts if prompt.is_deployed]


def create_prompt(
    project: str,
    prompt: str,
    version: Optional[int],
    input_variables: Optional[Dict[str, str]],
    model: str,
    provider: Optional[str],
    hyperparameters: Dict[str, Any],
    few_shot_examples: Optional[List[Dict[Any, Any]]],
) -> PromptResponse:
    """Create a prompt"""
    client = honeyhive_client()
    return client.create_prompt(
        prompt=PromptCreationQuery(
            task=project,
            prompt=prompt,
            version=version,
            input_variables=input_variables,
            model=model,
            provider=provider,
            hyperparameters=hyperparameters,
            few_shot_examples=few_shot_examples,
        )
    )


def update_prompt(
    id: str,
    version: int,
    input_variables: Dict[str, str],
    model: str,
    provider: Optional[str],
    hyperparameters: Dict[str, Any],
    is_deployed: bool,
    few_shot_examples: Optional[List[Dict[Any, Any]]],
) -> PromptResponse:
    """Update a prompt"""
    client = honeyhive_client()
    return client.update_prompt(
        prompt=PromptUpdateQuery(
            id=id,
            version=version,
            input_variables=input_variables,
            model=model,
            provider=provider,
            hyperparameters=hyperparameters,
            is_deployed=is_deployed,
            few_shot_examples=few_shot_examples,
        )
    )


def delete_prompt(id: str) -> DeleteResponse:
    """Delete a prompt"""
    client = honeyhive_client()
    return client.delete_prompt(id=id)


__all__ = ["get_prompts", "create_prompt", "update_prompt", "delete_prompt"]



from typing import Optional, Dict, List, Any
from pydantic import BaseModel, Field
import datetime


class PromptResponse(BaseModel):
    name: str = Field(
        title="Prompt name", description="The unique name of the prompt"
    )
    version: Optional[str] = Field(
        title="Version", description="The version of the prompt"
    )
    task: str = Field(
        title="Task",
        description="The task for which the prompt is being created",
    )
    text: str = Field(title="Text", description="The text of the prompt")
    input_variables: Optional[List[str]] = Field(
        title="Input Variables",
        description="The input variables to feed into the prompt",
    )
    model: str = Field(
        title="Model", description="The model to be used for the prompt"
    )
    hyperparameters: Dict[str, Any] = Field(
        title="Hyperparameters",
        description="The hyperparameters for the prompt",
    )
    is_deployed: Optional[bool] = Field(
        title="Is Deployed",
        description="The flag to indicate if the prompt is deployed",
    )
    few_shot_examples: Optional[List[Dict[Any, Any]]] = Field(
        title="Few Shot Examples",
        description="The few shot examples for the prompt, specified as a list of dictionaries"
        + " where each dictionary has a key-value pair of input variables and the corresponding"
        + " completion",
    )
    created_at: datetime.datetime
    updated_at: datetime.datetime


class PromptQuery(BaseModel):
    task: Optional[str] = Field(
        title="Task",
        description="The task for which the prompt is being created",
    )
    id: Optional[str] = Field(
        title="Prompt ID", description="The unique ID of the prompt"
    )


class ListPromptResponse(BaseModel):
    data: List[PromptResponse] = Field(
        title="Data", description="The list of prompts"
    )


class PromptCreationQuery(BaseModel):
    task: str = Field(
        title="Task",
        description="The task for which the prompt is being created",
    )
    text: str = Field(
        title="Text", description="The text of the prompt to be created"
    )
    version: Optional[int] = Field(
        title="Version", description="The version of the prompt"
    )
    input_variables: Optional[List[str]] = Field(
        title="Input Variables",
        description="The input variables to feed into the prompt",
    )
    model: str = Field(
        title="Model", description="The model to be used for the prompt"
    )
    provider: str = Field(
        title="Provider",
        description="The provider of the model to be used for the prompt",
    )
    hyperparameters: Dict[str, Any] = Field(
        title="Hyperparameters",
        description="The hyperparameters for the prompt",
    )
    few_shot_examples: Optional[List[Dict[Any, Any]]] = Field(
        title="Few Shot Examples",
        description="The few shot examples for the prompt, specified as a list of dictionaries"
        + " where each dictionary has a key-value pair of input variables and the corresponding"
        + " completion",
    )


class PromptUpdateQuery(BaseModel):
    id: str = Field(
        title="Prompt ID", description="The unique ID of the prompt"
    )
    version: int = Field(
        title="Version", description="The version of the prompt"
    )
    input_variables: List[str] = Field(
        title="Input Variables",
        description="The input variables to feed into the prompt",
    )
    model: str = Field(
        title="Model", description="The model to be used for the prompt"
    )
    provider: str = Field(
        title="Provider",
        description="The provider of the model to be used for the prompt",
    )
    hyperparameters: Dict[str, Any] = Field(
        title="Hyperparameters",
        description="The hyperparameters for the prompt",
    )
    is_deployed: bool = Field(
        title="Is Deployed",
        description="The flag to indicate if the prompt is deployed",
    )
    few_shot_examples: Optional[List[Dict[Any, Any]]] = Field(
        title="Few Shot Examples",
        description="The few shot examples for the prompt, specified as a list of dictionaries"
        + " where each dictionary has a key-value pair of input variables and the corresponding"
        + " completion",
    )



from typing import Dict, List, Optional, Any

from pydantic import BaseModel, Field


class EvaluationLoggingQuery(BaseModel):
    name: str = Field(
        title="Evaluation Name", description="The name of the evaluation"
    )
    task: str = Field(
        title="Project",
        description="The project for which the evaluation is being created",
    )
    prompts: List[Dict[str, Any]] = Field(
        title="Prompts",
        description="The prompts for which the evaluation is being created",
    )
    dataset: List[Dict[str, Any]] = Field(
        title="Dataset",
        description="The dataset for which the evaluation is being created",
    )
    datasetName: str = Field(
        title="Dataset Name",
        description="The name of the dataset for which the evaluation is being created",
    )
    summary: Optional[List[Dict[str, Any]]] = Field(
        title="Summary", description="The metric summary of the evaluation"
    )
    results: List[Dict[str, Any]] = Field(
        title="Results", description="The results of the evaluation"
    )
    accepted: List[List[Any]] = Field(
        title="Accepted",
        description="The accepted completions for the prompts that are being evaluated",
    )
    metrics: List[List[Dict[str, Any]]] = Field(
        title="Metrics",
        description="The metrics that are computed for the completions",
    )
    metrics_to_compute: List[str] = Field(
        title="Metrics to Compute",
        description="The metrics that are computed for the completions",
    )
    comments: List[Dict[str, Any]] = Field(
        title="Comments", description="The comments for the evaluation"
    )
    generations: Optional[List[Any]] = Field(
        title="Generations",
        description="The completions for prompts that are being evaluated",
    )
    description: Optional[str] = Field(
        title="Evaluation Description",
        description="The description of the evaluation",
    )


class SuccessResponse(BaseModel):
    success: bool = Field(
        title="Success", description="Whether the request was successful"
    )
    id: Optional[str] = Field(
        title="ID", description="The ID of the object that was created"
    )



from typing import Dict, List, Optional, Any
from honeyhive.api.models.generations import (
    GenerateQuery,
    GenerationsQuery,
    GenerationResponse,
    ListGenerationResponse,
    GenerationLoggingQuery,
)
from honeyhive.sdk.init import honeyhive_client
import pandas as pd


def generate(
    project: str,
    input: Dict[str, str],
    prompts: Optional[List[str]] = None,
    model_id: Optional[str] = None,
    best_of: Optional[int] = None,
    metric: Optional[str] = None,
    sampling: Optional[str] = None,
    user_properties: Optional[Dict[str, Any]] = None,
    source: Optional[str] = None,
    stream: Optional[bool] = False
) -> GenerationResponse:
    """Generate completions"""
    client = honeyhive_client()

    if stream == False:
        return client.generate(
            generation=GenerateQuery(
                task=project,
                input=input,
                prompts=prompts,
                model_id=model_id,
                best_of=best_of,
                metric=metric,
                sampling=sampling,
                user_properties=user_properties,
                source=source,
            )
        )
    else:
        return client.generate_stream(
            generation=GenerateQuery(
                task=project,
                input=input,
                prompts=prompts,
                model_id=model_id,
                best_of=best_of,
                metric=metric,
                sampling=sampling,
                user_properties=user_properties,
                source=source,
                stream=stream
            )
        )


def get_generations(project: Optional[str] = None, query: Optional[Dict[str, Any]] = None, limit: Optional[int] = None):
    """Get all generations"""
    client = honeyhive_client()
    generations_list = client.get_generations(
        query=GenerationsQuery(
            task=project
        )
    )
    
    if query == None:
        query = {}
    
    print(generations_list["pagination"])

    # go through the pages and get all the generations
    while generations_list["pagination"]["nextPage"] != None:
        query["page"] = generations_list["pagination"]["nextPage"]
        page=query["page"]
        #print("page", page)
        next_page = client.get_generations(
            query=GenerationsQuery(
                task=project,
                page=page
            )
        )
        generations_list["generations"] += next_page["generations"]
        generations_list["pagination"]["nextPage"] = next_page["pagination"]["nextPage"]
        #print(generations_list["pagination"])

        if limit != None and len(generations_list["generations"]) >= limit:
            break
    
    import pandas as pd

    df = pd.DataFrame(generations_list["generations"])
    return df


def log(
    project: str,
    model: str,
    generation: str,
    hyperparameters: Dict[str, Any],
    source: Optional[str] = None,
    version: Optional[str] = None,
    prompt_template: Optional[str] = None,
    inputs: Optional[Dict[str, Any]] = None,
    prompt: Optional[str] = None,
    ground_truth: Optional[str] = None,
    usage: Optional[Dict[str, Any]] = None,
    latency: Optional[float] = None,
    user_properties: Optional[Dict[str, Any]] = None,
    metadata: Optional[Dict[str, Any]] = None,
    feedback: Optional[Dict[str, Any]] = None,
) -> GenerationResponse:
    # log generation via honeyhive
    client = honeyhive_client()

    if prompt == None:
        prompt = prompt_template
        if inputs == None and prompt_template == None:
            # throw an exception
            return Exception(
                "Please provide either a complete prompt or prompt template with inputs"
            )

    honeyhive_response = client.log_generation(
        generation=GenerationLoggingQuery(
            task=project,
            version=version,
            model=model,
            prompt=prompt,
            inputs=inputs,
            hyperparameters=hyperparameters,
            generation=generation,
            usage=usage,
            latency=latency,
            source=source,
            user_properties=user_properties,
            metadata=metadata,
            ground_truth=ground_truth,
            feedback=feedback,
        )
    )

    return honeyhive_response


__all__ = ["generate", "log", "get_generations"]



from .fine_tuned_models import FineTunedModelResponse
from .prompts import PromptResponse
from .datasets import DatasetResponse
from .metrics import MetricResponse

from typing import List, Optional, Any, Dict
from pydantic import BaseModel, Field
import datetime


class TaskResponse(BaseModel):
    id: str = Field(title="Task ID", description="The unique ID of the task")
    name: str = Field(title="Task Name", description="The name of the task")
    type: Optional[str] = Field(
        title="Task Type", description="The type of the task"
    )
    fine_tuned_models: Optional[List[FineTunedModelResponse]]
    prompts: List[PromptResponse]
    datasets: Optional[List[DatasetResponse]]
    metrics: Optional[List[MetricResponse]]
    created_at: datetime.datetime
    updated_at: datetime.datetime


class TaskCreationQuery(BaseModel):
    name: str = Field(title="Task Name", description="The name of the task")
    type: Optional[str] = Field(
        title="Task Type", description="The type of the task"
    )
    fine_tuned_models: Optional[List[FineTunedModelResponse]] = Field(
        title="Fine Tuned Models",
        description="The fine tuned models for the task",
    )
    prompts: Optional[List[PromptResponse]] = Field(
        title="Prompts", description="The prompts for the task"
    )
    datasets: Optional[List[DatasetResponse]] = Field(
        title="Datasets", description="The datasets for the task"
    )
    metrics: Optional[List[MetricResponse]] = Field(
        title="Metrics", description="The metrics for the task"
    )


class TaskUpdateQuery(BaseModel):
    name: Optional[str] = Field(
        title="Task Name", description="The name of the task"
    )
    type: Optional[str] = Field(
        title="Task Type", description="The type of the task"
    )
    fine_tuned_models: Optional[List[FineTunedModelResponse]] = Field(
        title="Fine Tuned Models",
        description="The fine tuned models for the task",
    )
    prompts: Optional[List[PromptResponse]] = Field(
        title="Prompts", description="The prompts for the task"
    )
    datasets: Optional[List[DatasetResponse]] = Field(
        title="Datasets", description="The datasets for the task"
    )
    metrics: Optional[List[MetricResponse]] = Field(
        title="Metrics", description="The metrics for the task"
    )


class ListTaskResponse(BaseModel):
    data: List[TaskResponse] = Field(
        title="List of tasks", description="The list of tasks"
    )



from honeyhive.api.models.evaluations import EvaluationLoggingQuery
import honeyhive
from typing import Dict, List, Any
from honeyhive.sdk.generations import log as log_generation
import pandas as pd
import random
import json

class HoneyHiveEvaluation:
    def __init__(self, project: str, name: str, dataset_name: str = "", description: str = "", metrics: List[str] = []):
        self.project = project
        self.name = name
        self.dataset_name = dataset_name
        self.description = description
        self.metrics = metrics
        self.configs = []
        self.inputs = []
        self.run = []
        self.completions = []
        self.logged_metrics = []
        self.summary = []
        self.accepted = []
        self.comments = []
        # create a hash set from input, config pair to is_logged
        # if input, config pair is in hash set, then it is logged
        self.is_logged = []
    
    def log_run(
        self,
        config: Dict[str, Any],
        input: Dict[str, Any],
        completion: str,
        metrics: Dict[str, Any],
        ground_truth: str = None,
        metadata: Dict[str, Any] = None,
        accepted: bool = None):
        """Log a single run"""
        # check if config is already in self.configs
        # if it is, get the index of it
        # if it isn't, add it to self.configs and get the index of it
        if config not in self.configs:
            self.configs.append(config)
        config_index = self.configs.index(config)

        # check if input is already in self.inputs
        # if it is, get the index of it
        # if it isn't, add it to self.inputs and get the index of it
        if ground_truth is not None:
            input["ground_truth"] = ground_truth
        if len(self.configs) <= 1:
            self.inputs.append(input)
        else:
            # check how many tuples have input as the input
            is_logged = 0
            for log in self.is_logged:
                if json.dumps(input) == log[1]:
                    is_logged += 1
            modulo = is_logged % len(self.configs)
            #print("is_logged", is_logged)
            #print("modulo", modulo)
            if input not in self.inputs or modulo == 0:
                self.inputs.append(input)
        res_list = [i for i, value in enumerate(self.inputs) if value == input]
        input_index = res_list[-1]

        self.is_logged.append((json.dumps(config), json.dumps(input)))
        # add completion to self.completions at config_index, input_index
        if len(self.completions) <= config_index:
            self.completions.append([])
        if len(self.completions[config_index]) <= input_index:
            self.completions[config_index].append([])
        #print("inputs length", len(self.inputs))
        #print("configs length", len(self.configs))
        #print("input_index", input_index)
        #print("config_index", config_index)
        #print("completions length", len(self.completions))
        #print("completions[config_index] length", len(self.completions[config_index]))
        self.completions[config_index][input_index] = completion

        if metadata is None:
            metadata = {}
        
        # merge metrics and metadata
        metrics = {**metrics, **metadata}

        generation_response = log_generation(
            project=self.project,
            model=config["model"],
            generation=completion,
            hyperparameters=config["hyperparameters"] if "hyperparameters" in config else {},
            source="evaluation",
            version=config["name"] if "name" in config else "",
            prompt_template=config["chat"][0]["content"] if "chat" in config else [],
            inputs=input,
            ground_truth=ground_truth,
            latency=metrics["latency"] if "latency" in metrics else None,
            user_properties=config["user_properties"] if "user_properties" in config else {},
            metadata=metrics,
            feedback={
                "accepted": accepted
            }
        )
        
        generation_id = generation_response.generation_id

        metrics["generation_id"] = generation_id

        # add metrics to self.logged_metrics at input_index, config_index
        if len(self.logged_metrics) <= input_index:
            self.logged_metrics.append([])
        self.logged_metrics[input_index].append(metrics)
        # #print("input index", input_index)
        # #print(self.logged_metrics)

        # add accepted to self.accepted at config_index, input_index
        if len(self.accepted) <= config_index:
            self.accepted.append([])
        if len(self.accepted[config_index]) <= input_index:
            self.accepted[config_index].append([None])
        self.accepted[config_index][input_index] = accepted


    def log_comment(self, comment: str):
        self.comments.append({"text": comment})
    
    def finish(self):
        for config in self.configs:
            if "name" not in config:
                # generate random 5 character string
                config["name"] = ''.join(
                    random.choice('0123456789ABCDEF') for i in range(5)
                )
                config["version"] = config["name"]
                config["task"] = self.project
        
        summary = []

        # get the keys of metrics[0][0]
        # for each key, get the average of that key
        # add that to summary
        #print(self.logged_metrics)
        if len(self.logged_metrics) == 0 or len(self.logged_metrics[0]) == 0:
            print("Warning: no metrics logged")
            keys = []
        else:
            keys = list(self.logged_metrics[0][0].keys())
        # #print(self.logged_metrics)
        # #print(len(self.logged_metrics))
        keys.append("Positive Feedback (%)")
        for key in keys:
            summary_entry = {"metric": key}

            for config in self.configs:
                summary_entry[config["name"]] = 0
            
            if key != "Positive Feedback (%)" and key != "session_id" and key != "generation_id":
                for input_index in range(len(self.inputs)):
                    for config_index in range(len(self.configs)):
                        try:
                            summary_entry[self.configs[config_index]["name"]] += self.logged_metrics[input_index][config_index][key]
                        except Exception as e:
                            print("Warning: metric", key, " for input ", input_index, " & config ", config_index, " is invalid")
                            pass

            for config in self.configs:
                summary_entry[config["name"]] /= len(self.logged_metrics)

            summary.append(summary_entry)
        
        self.summary = summary

        results = []

        i = 0
        j = 0

        #print(self.completions)
        for i in range(len(self.inputs)):
            single_result = {}
            j = 0
            for config in self.configs:
                ##print("i", i, "j", j)
                single_result[config["name"]] = self.completions[j][i]
                self.logged_metrics[i][j]["prompt"] = config["name"]
                j += 1
            results.append(single_result)
        
        #print(results)
        from honeyhive.sdk.init import honeyhive_client
        client = honeyhive_client()

        response = client.log_evaluation(
            evaluation=EvaluationLoggingQuery(
                name=self.name,
                task=self.project,
                prompts=self.configs,
                dataset=self.inputs,
                datasetName=self.dataset_name,
                generations=self.completions,
                metrics=self.logged_metrics,
                summary=summary,
                comments=self.comments,
                accepted=self.accepted,
                description=self.description,
                results=results,
                metrics_to_compute=self.metrics
            )
        )

        url = "https://app.honeyhive.ai/evaluate?id="
        print("Evaluation logged successfully")
        print("View evaluation at", url + response.id)


def init(
    project: str,
    name: str = None,
    dataset_name: str = None,
    description: str = None,
    metrics: List[str] = None,
):
    """Initialize an evaluation"""
    import uuid

    # TODO check if project exists
    try:
        is_valid = honeyhive.projects.get(project)
        ##print(is_valid)
    except:
        raise Exception("Project does not exist")
    
    return {
        "uuid": str(uuid.uuid4()),
        "project": project,
        "name": name,
        "dataset_name": dataset_name if dataset_name is not None else "",
        "description": description if description is not None else "",
        "metrics": metrics if metrics is not None else [],
    }


def log(
    evaluation: Dict[str, Any],
    configs: List[Dict[str, Any]],
    inputs: List[Dict[str, Any]],
    run: List[List[Dict[str, Any]]],
    completions: List[List[str]] = None,
    metrics: List[List[Dict[str, Any]]] = None,
    accepted: List[List[bool]] = None,
    comments: List[str] = None,
):
    """Log an evaluation"""
    from honeyhive.sdk.init import honeyhive_client
    import random

    client = honeyhive_client()

    import copy

    run = copy.deepcopy(run)

    new_comments = []

    for comment in comments:
        new_comments.append({"text": comment})

    comments = new_comments

    if run:
        # completions is the whole list of list of completions
        # metrics is the whole list of list of metrics except "completion"
        completions = []
        metrics = []
        for i in range(len(run)):
            completion_run = []
            metric_calculation = []
            for j in range(len(run[i])):
                rest_run = run[i][j]
                completion_run.append(rest_run["completion"])
                del rest_run["completion"]
                rest_run["Cost ($)"] = rest_run["cost"]
                del rest_run["cost"]
                rest_run["Latency (ms)"] = rest_run["latency"]
                del rest_run["latency"]
                rest_run["Response Length (tokens)"] = rest_run[
                    "response_length"
                ]
                del rest_run["response_length"]
                metric_calculation.append(rest_run)
            completions.append(completion_run)
            metrics.append(metric_calculation)
    # check if configs have names
    # if they don't, give them random 5 character string
    for config in configs:
        if "name" not in config:
            # generate random 5 character string
            config["name"] = ''.join(
                random.choice('0123456789ABCDEF') for i in range(5)
            )
            config["version"] = config["name"]
            config["task"] = evaluation['project']

    results = []
    summary = []

    # get the keys of metrics[0][0]
    # for each key, get the average of that key
    # add that to summary
    keys = metrics[0][0].keys()
    for key in keys:
        summary_entry = {"metric": key}

        for config in configs:
            summary_entry[config["name"]] = 0

        for i in range(len(metrics)):
            for j in range(len(metrics[i])):
                try:
                    summary_entry[configs[j]["name"]] += metrics[i][j][key]
                except Exception as e:
                    print("Warning: metric", key, " for input ", i, " & config ", j, " is invalid")
                    pass

        for config in configs:
            summary_entry[config["name"]] /= len(metrics)

        summary.append(summary_entry)

    i = 0
    j = 0
    for input in inputs:
        single_result = {}
        j = 0
        for config in configs:
            single_result[config["name"]] = completions[i][j]
            metrics[i][j]["prompt"] = config["name"]
            j += 1
        i += 1
        results.append(single_result)

    response = client.log_evaluation(
        evaluation=EvaluationLoggingQuery(
            name=evaluation['name'],
            task=evaluation['project'],
            prompts=configs,
            dataset=inputs,
            datasetName=evaluation['dataset_name']
            if evaluation['dataset_name'] is not None
            else "",
            generations=completions,
            metrics=metrics,
            summary=summary,
            comments=comments,
            accepted=accepted,
            description=evaluation['description']
            if evaluation['description'] is not None
            else "",
            results=results,
            metrics_to_compute=evaluation['metrics'],
        )
    )
    url = "https://app.honeyhive.ai/evaluate?id="
    print("Evaluation logged successfully")
    print("View evaluation at", url + response.id)

    return response


def log_from_df(
    project: str,
    df: pd.DataFrame,
    name: str = None,
    dataset_name: str = None,
    description: str = None,
    accepted: List[List[bool]] = None,
    comments: List[Dict[str, Any]] = None,
):
    """Log an evaluation from a dataframe"""
    from honeyhive.sdk.init import honeyhive_client

    client = honeyhive_client()
    return client.log_evaluation(
        evaluation=EvaluationLoggingQuery(
            name=name,
            task=project,
            prompts=df["configs"].tolist(),
            dataset=df["inputs"].tolist(),
            dataset_name=dataset_name,
            generations=df["completions"].tolist(),
            metrics=df["metrics"].tolist(),
            comments=comments,
            description=description,
            results=accepted,
        )
    )

def get_evaluation(
    id: str
):
    """Get an evaluation"""
    from honeyhive.sdk.init import honeyhive_client

    client = honeyhive_client()
    return client.get_evaluation(
        id=id
    )


__all__ = [
    "init",
    "log",
    "log_from_df",
    "HoneyHiveEvaluation",
    "get_evaluation"
]



from typing import List, Optional, Any, Dict
from pydantic import BaseModel, Field


class ChatQuery(BaseModel):
    project: str = Field(
        title="Project",
        description="The project that the chat is associated with",
    )
    version: Optional[str] = Field(
        title="Version",
        description="The version of the chat template",
    )
    inputs: Optional[Dict[str, Any]] = Field(
        title="Inputs",
        description="The inputs to fill into the chat template",
    )
    provider: str = Field(
        title="Provider",
        description="The provider that the chat is associated with",
    )
    model: str = Field(
        title="Model",
        description="The model that is used to generate the text",
    )
    hyperparameters: Optional[Dict[str, Any]] = Field(
        title="Hyperparameters",
        description="The hyperparameters that are used to generate the text",
    )
    messages: List[Dict[str, Any]] = Field(
        title="Messages",
        description="The messages that are used to generate the text",
    )
    functions: Optional[List[Dict[str, Any]]] = Field(
        title="Functions",
        description="The functions that are provided to the model to use",
    )
    function_call: Optional[str] = Field(
        title="Function Call",
        description="The function call mode that is used to generate the text",
    )
    plugins: Optional[List[str]] = Field(
        title="Plugins",
        description="The plugins that are used to generate the text",
    )
    source: Optional[str] = Field(
        title="Source",
        description="The source of the chat completion",
    )
    num_samples: Optional[int] = Field(
        title="Number of Samples",
        description="The number of samples to generate",
    )
    stream: Optional[bool] = Field(
        title="Streaming",
        description="Whether the chat completion is to be streamed",
    )
    user: Optional[str] = Field(
        title="User",
        description="The user that generated the text",
    )
    metadata: Optional[Dict[str, Any]] = Field(
        title="Metadata",
        description="The metadata to track for the chat completion",
    )
    ground_truth: Optional[str] = Field(
        title="Ground Truth",
        description="The ground truth for the chat completion",
    )

class ChatResponse(BaseModel):
    timestamp: str = Field(
        title="Timestamp",
        description="The timestamp of the chat completion",
    )
    generation_id: str = Field(
        title="Generation ID",
        description="The unique ID of the generation",
    )
    generation: str = Field(
        title="Generation",
        description="The text that was generated by your model provider",
    )
    metrics: Optional[Dict[str, Any]] = Field(
        title="Metrics",
        description="The metrics associated with the generation",
    )
    completion_tokens: int = Field(
        title="Completion Tokens",
        description="The number of tokens generated for the completion",
    )
    choices: List[Dict[str, Any]] = Field(
        title="Choices",
        description="The choices associated with the generation",
    )
    cost: float = Field(
        title="Cost",
        description="The cost of the generation",
    )
    latency: float = Field(
        title="Latency",
        description="The latency of the generation in milliseconds",
    )


from typing import Dict, List, Optional, Any

from pydantic import BaseModel, Field

class SessionStartQuery(BaseModel):
    session_id: str = Field(
        title="Session ID",
        description="The ID of the session"
    )
    project: str = Field(
        title="Project",
        description="The project name for the session"
    )
    source: str = Field(
        title="Source",
        description="The source of the session"
    )
    session_name: Optional[str] = Field(
        title="Session Name",
        description="The name for the session"
    )
    session_type: Optional[str] = Field(
        title="Session Type",
        description="The type for the session, default is chain"
    )
    user_properties: Optional[Dict] = Field(
        title="User Properties",
        description="The user properties for the user starting the session"
    )
    metadata: Optional[Dict] = Field(
        title="Metadata",
        description="Metadata for the session"
    )

class SessionStartResponse(BaseModel):
    session_id: str = Field(
        title="Session ID",
        description="The ID of the started session"
    )
    project_id: str = Field(
        title="Project ID",
        description="The ID of the project for the session"
    )

class SessionEndResponse(BaseModel):
    success: bool = Field(
        title="Success",
        description="Whether ending the session was successful"
    )

class SessionEventQuery(BaseModel):
    created_at: str = Field(
        title="Created At",
        description="The time the event was created"
    )
    session_id: str = Field(
        title="Session ID",
        description="The ID of the session"
    )
    event_id: str = Field(
        title="Event ID",
        description="The ID of the event"
    )
    parent_id: Optional[Any] = Field(
        title="Parent ID",
        description="The parent ID of the event"
    )
    event_type: str = Field(
        title="Event Type",
        description="The type of the event"
    )
    event_name: str = Field(
        title="Event Name",
        description="The name for the event"
    )
    config: Dict = Field(
        title="Config",
        description="The configuration of LLM, Tool or other for the event"
    )
    children: Optional[List] = Field(
        title="Children",
        description="Child events"
    )
    inputs: Dict = Field(
        title="Inputs",
        description="Inputs to the event"
    )
    output: str = Field(
        title="Output", 
        description="Output of the event"
    )
    error: Optional[Any] = Field(
        title="Error",
        description="Error from the event"
    )
    source: Optional[str] = Field(
        title="Source",
        description="Source of the event"
    )
    duration: Optional[float] = Field(
        title="Duration",
        description="Duration of the event"
    )
    event_properties: Optional[Dict] = Field(
        title="Event Properties",
        description="Properties of the event"
    )
    user_properties: Optional[Dict] = Field(
        title="User Properties",
        description="User properties for the event"
    )
    metrics: Optional[Dict] = Field(
        title="Metrics",
        description="Metrics for the event"
    )
    feedback: Optional[Dict] = Field(
        title="Feedback",
        description="Feedback for the event"
    )
    metadata: Optional[Dict] = Field(
        title="Metadata",
        description="Metadata for the event"
    )

class SessionEventResponse(BaseModel):
    success: bool = Field(
        title="Success",
        description="Whether recording the event was successful"
    )
    event_id: str = Field(
        title="Event ID",
        description="The ID of the event"
    )

class SessionFeedback(BaseModel):
    feedback: Dict[str, Any] = Field(
        title="Feedback",
        description="The feedback for the session"
    )
    event_id: Optional[str] = Field(
        title="Event ID",
        description="The ID of the specific event to provide feedback for"
    )
    ground_truth: Optional[str] = Field(
        title="Ground Truth",
        description="The ground truth for the event/session"
    )

class SessionQuery(BaseModel):
    project: str = Field(
        title="Project",
        description="The project to query sessions for"
    )
    query: Dict = Field(
        title="Query",
        description="The query for finding sessions"
    )
    limit: Optional[int] = Field(  
        title="Limit",
        description="The maximum number of sessions to return"
    )

class SessionTrace(BaseModel):
    logs: List[SessionEventQuery] = Field(
        title="Logs",
        description="The trace logs for the session"
    )


from typing import Optional, List
from pydantic import BaseModel, Field
import datetime


class FineTunedModelResponse(BaseModel):
    id: str = Field(
        title="Fine Tuned Model ID",
        description="The unique ID of the fine tuned model",
    )
    object: str = Field(
        title="Object", description="The object type of the fine tuned model"
    )
    model: str = Field(
        title="Model", description="The model used for fine tuning"
    )
    created_at: datetime.datetime = Field(
        title="Created At",
        description="The date and time when the fine tuned model was created",
    )
    updated_at: datetime.datetime = Field(
        title="Updated At",
        description="The date and time when the fine tuned model was last updated",
    )
    fine_tuned_model: Optional[str] = Field(
        title="Fine Tuned Model", description="The fine tuned model"
    )
    hyperparams: Optional[str] = Field(
        title="Hyperparameters",
        description="The hyperparameters used for fine tuning",
    )
    org_id: str = Field(
        title="Organization ID",
        description="The unique ID of the organization",
    )
    result_files: Optional[str] = Field(
        title="Result Files",
        description="The result files of the fine tuned model",
    )
    status: str = Field(
        title="Status", description="The status of the fine tuned model"
    )
    validation_files: Optional[str] = Field(
        title="Validation Files",
        description="The validation files used for fine tuning",
    )
    training_files: Optional[str] = Field(
        title="Training Files",
        description="The training files used for fine tuning",
    )


class ListFineTunedModelResponse(BaseModel):
    data: List[FineTunedModelResponse] = Field(
        title="Data", description="The list of fine tuned models"
    )



from honeyhive.api.client import HoneyHive, get_client
from typing import Dict, Optional
import honeyhive


def honeyhive_client() -> HoneyHive:
    """Get a HoneyHive client"""
    return get_client("https://api.honeyhive.ai", honeyhive.api_key)



from typing import Optional, Any, Dict
from pydantic import BaseModel, Field


class DeleteResponse(BaseModel):
    deleted: bool = Field(
        title="Deleted",
        description="The boolean value indicating whether the object was deleted or not",
    )
